<?php

/**
 * Licentia Fidelitas - SMS Notifications for E-Goi
 *
 * NOTICE OF LICENSE
 * This source file is subject to the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International
 * It is available through the world-wide-web at this URL:
 * http://creativecommons.org/licenses/by-nc-nd/4.0/
 *
 * @title      SMS Notifications
 * @category   Marketing
 * @package    Licentia
 * @author     Bento Vilas Boas <bento@licentia.pt>
 * @copyright  Copyright (c) 2016 E-Goi - http://e-goi.com
 * @license    Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International
 */
class Licentia_Fidelitas_Model_Autoresponders extends Mage_Core_Model_Abstract
{

    /**
     *
     */
    const MYSQL_DATE = 'yyyy-MM-dd';
    /**
     *
     */
    const MYSQL_DATETIME = 'yyyy-MM-dd HH:mm:ss';

    /**
     *
     */
    protected function _construct()
    {

        $this->_init('fidelitas/autoresponders');
    }

    /**
     * @return array
     */
    public function toOptionArray()
    {

        $return = array(
            'order_new'      => Mage::helper('fidelitas')->__('Order - New Order'),
            #'order_product'  => Mage::helper('fidelitas')->__('Order - Bought Specific Product'),
            'order_status'   => Mage::helper('fidelitas')->__('Order - Order Status Changes'),
            'new_invoice'    => Mage::helper('fidelitas')->__('New Invoice'),
            'new_shipment'   => Mage::helper('fidelitas')->__('New Shipment'),
            'new_creditmemo' => Mage::helper('fidelitas')->__('New Creditmemo'),
        );


        return $return;
    }

    /**
     * @param $event
     *
     * @return bool
     */
    public function newOrderDocument($event)
    {

        try {
            $document = $event->getEvent()->getDataObject();

            if ($document instanceof Mage_Sales_Model_Order_Invoice) {
                $type = 'new_invoice';
            } elseif ($document instanceof Mage_Sales_Model_Order_Shipment) {
                $type = 'new_shipment';
            } elseif ($document instanceof Mage_Sales_Model_Order_Creditmemo) {
                $type = 'new_creditmemo';
            } else {
                return false;
            }

            $order = $document->getOrder();

            $phone = Mage::getModel('fidelitas/egoi')->getPhone($order);

            if (!$phone) {
                return false;
            }

            $autoresponders = $this->_getCollection($order->getStoreId())
                ->addFieldToFilter('event', $type);

            $customer = new Varien_Object;
            $customer->setName($order->getCustomerName())
                ->setEmail($order->getCustomerEmail())
                ->setId($order->getCustomerId());

            foreach ($autoresponders as $autoresponder) {
                $this->_insertData($autoresponder, $phone, $order->getStoreId(), $customer, $document->getId());
                Mage::log('OK - DOCUMENT ' . get_class($document) . ':  Phone: ' . $phone . ' Order: ' . $order->getId(), null, 'egoi.sms.txt', true);
            }
        } catch (Exception $exception) {

        }
    }

    /**
     * @param $event
     *
     * @return bool|void
     */
    public function changeStatus($event)
    {
        try {
            $order = $event->getEvent()->getOrder();
            $newStatus = $order->getData('status');
            $olderStatus = $order->getOrigData('status');

            if ($newStatus == $olderStatus) {
                Mage::log('Ignoring: Equal Status. ' . $order->getId(), null, 'egoi.sms.txt', true);
                return;
            }

            $phone = Mage::getModel('fidelitas/egoi')->getPhone($order);

            if (!$phone) {
                return false;
            }

            $autoresponders = $this->_getCollection($order->getStoreId())
                ->addFieldToFilter('event', 'order_status')
                ->addFieldToFilter('order_status', $newStatus);

            $customer = new Varien_Object;
            $customer->setName($order->getCustomerName())
                ->setEmail($order->getCustomerEmail())
                ->setId($order->getCustomerId());

            foreach ($autoresponders as $autoresponder) {
                $this->_insertData($autoresponder, $phone, $order->getStoreId(), $customer, $order->getId());
                Mage::log('OK - CHANGE STATUS:  Phone: ' . $phone . ' Order: ' . $order->getId(), null, 'egoi.sms.txt', true);
            }
        } catch (Exception $exception) {

        }
    }

    /**
     * @param      $event
     * @param null $redirect
     *
     * @return bool
     */
    public function newOrder($event, $redirect = null)
    {
        try {
            $message = null;

            if (!$redirect) {
                $order = $event->getEvent()->getOrder();
            } else {
                $order = $event;
            }

            if ($order->getPayment()->getMethod() == 'pagseguro_boleto' && !$redirect) {
                Mage::getSingleton('core/session')->setData('egoi_pagseguro', $order->getId());
                return $this;
            }

            if ($order->getPayment()->getMethod() == 'pagseguro_boleto' && $redirect) {
                $order = $event;
                $message = $redirect;
            }
            $autoresponders = $this->_getCollection($order->getStoreId())
                ->addFieldToFilter('event', array('in' => array('order_product', 'order_new')));

            $customer = new Varien_Object;
            $customer->setName($order->getCustomerName())
                ->setEmail($order->getCustomerEmail())
                ->setId($order->getCustomerId());

            foreach ($autoresponders as $autoresponder) {

                if ($autoresponder->getShippingMethod()) {
                    $allow = explode(',', $autoresponder->getShippingMethod());

                    $shippingMethod = $order->getData('shipping_method');
                    if (!in_array($shippingMethod, $allow)) {
                        continue;
                    }
                }
                if ($autoresponder->getPaymentMethod()) {
                    $allow = explode(',', $autoresponder->getPaymentMethod());
                    $paymentMethod = $order->getPayment()->getMethod();

                    if (!in_array($paymentMethod, $allow)) {
                        continue;
                    }
                }

                if ($autoresponder->getEvent() == 'order_product') {
                    $items = $order->getAllItems();
                    $ok = false;
                    foreach ($items as $item) {
                        if ($item->getProductId() == $autoresponder->getProduct()) {
                            $ok = true;
                            break;
                        }
                    }
                    if ($ok === false) {
                        break;
                    }
                }

                $phone = Mage::getModel('fidelitas/egoi')->getPhone($order);

                if (!$phone) {
                    return false;
                }

                $this->_insertData($autoresponder, $phone, $order->getStoreId(), $customer, $order->getId(), $message, $redirect);
                Mage::log('OK - NEW ORDER:  Phone: ' . $phone . ' Order: ' . $order->getId(), null, 'egoi.sms.txt', true);
            }

        } catch (Exception $exception) {

        }
    }

    /**
     * @param $autoresponder
     *
     * @return string|Zend_Date
     * @throws Zend_Date_Exception
     */
    public function calculateSendDate($autoresponder)
    {
        if ($autoresponder->getSendMoment() == 'occurs') {
            $date = Mage::app()->getLocale()->date()
                ->get(self::MYSQL_DATETIME);
        }

        if ($autoresponder->getSendMoment() == 'after') {
            $date = Mage::app()->getLocale()->date();

            if ($autoresponder->getAfterHours() > 0) {
                $date->addHour($autoresponder->getAfterHours());
            }
            if ($autoresponder->getAfterDays() > 0) {
                $date->addDay($autoresponder->getAfterDays());
            }
            $date->get(self::MYSQL_DATETIME);
        }

        return $date;
    }

    /**
     *
     */
    public function send()
    {
        $date = Mage::app()->getLocale()->date()->get(self::MYSQL_DATETIME);

        $smsCollection = Mage::getModel('fidelitas/events')->getCollection()
            ->addFieldToFilter('sent', 0)
            ->addFieldToFilter('send_at', array('lteq' => $date));

        foreach ($smsCollection as $cron) {

            if (!Mage::getModel('fidelitas/egoi')->validateNumber($cron->getCellphone())) {
                $cron->delete();
                continue;
            }

            $autoresponder = Mage::getModel('fidelitas/autoresponders')->load($cron->getAutoresponderId());
            $messageProcessor = Mage::helper('cms')->getBlockTemplateProcessor();

            $order = false;
            if ($cron->getDataObjectId()) {
                /** @var Mage_Sales_Model_Order $order */
                $order = Mage::getModel('sales/order')->load($cron->getDataObjectId());
                $variables = array();
                $variables['order'] = $order;
                $messageProcessor->setVariables($variables);
            }

            $message = $messageProcessor->filter($autoresponder->getMessage());

            $replaceMb = false;
            if ($order && $order->getPayment()->getMethod() == 'multibanco') {
                $replaceMb = true;
                $entidade = $order->getPayment()->getAdditionalInformation('entidade');
                $referencia = $order->getPayment()->getAdditionalInformation('referencia');
                $valor = $order->getPayment()->getAdditionalInformation('valor');
            }

            if ($order && $order->getPayment()->getMethod() == 'ifmb') {
                $replaceMb = true;
                $entidade = $order->getPayment()->getData('ifthenpay_entidade');
                $referencia = $order->getPayment()->getData('ifthenpay_referencia');
                $valor = $order->getPayment()->getData('ifthenpay_montante');
            }

            if ($order && $order->getPayment()->getMethod() == 'pagseguro_boleto') {
                $message = str_replace('{boleto}', Mage::getUrl('egoi/r/i', array('c' => md5($cron->getExtra()))), $message);
            }

            if ($replaceMb) {

                $message = str_replace(['{entidade}', '{referencia}', '{valor}'], [$entidade, $referencia, number_format($valor, 2)], $message);
            }

            if ($autoresponder->getEvent() == 'new_shipment') {

                $track = Mage::getModel('sales/order_shipment')
                    ->load($cron->getDataObjectId())
                    ->getTracksCollection()
                    ->getFirstItem();

                if ($track->getId()) {
                    $message = str_replace(
                        array('{track_number}', '{track_title}'),
                        array($track->getTrackNumber(), $track->getTitle()),
                        $message
                    );
                }
            }

            try {
                $result = Mage::getModel('fidelitas/egoi')->send($cron->getCellphone(), $message);
            } catch (Exception $exception) {
                $result = false;
            }

            if ($result === true) {
                $cron->setSent(1)->setMessage($message)->setSentAt($date)->save();
            } else {
                $datetime1 = new \DateTime($date);
                $datetime2 = new \DateTime($cron->getData('send_at'));
                $interval = $datetime1->diff($datetime2);
                if ($interval->format('%h') > 3) {
                    $cron->delete();
                }

            }

        }
    }

    /**
     * @param      $autoresponder
     * @param      $number
     * @param      $storeId
     * @param      $customer
     * @param null $dataObjectId
     * @param null $message
     * @param null $extra
     *
     * @return bool
     * @throws Zend_Date_Exception
     */
    protected function _insertData($autoresponder, $number, $storeId, $customer, $dataObjectId = null, $message = null, $extra = null)
    {

        $storeIds = explode(',', $autoresponder->getStoreIds());

        if (!in_array($storeId, $storeIds) && !in_array(0, $storeIds)) {
            return false;
        }

        $data = array();
        $data['send_at'] = $this->calculateSendDate($autoresponder);
        $data['autoresponder_id'] = $autoresponder->getId();
        $data['cellphone'] = $number;
        $data['customer_id'] = $customer->getId();
        $data['customer_name'] = $customer->getName();
        $data['customer_email'] = $customer->getEmail();
        $data['event'] = $autoresponder->getEvent();
        $data['created_at'] = new Zend_Db_Expr('NOW()');
        $data['sent'] = 0;
        $data['data_object_id'] = $dataObjectId;
        $data['message'] = $message;
        $data['extra'] = $extra;

        Mage::getModel('fidelitas/events')->setData($data)->save();
        $autoresponder->setData('number_subscribers', $autoresponder->getData('number_subscribers') + 1)->save();
    }

    /**
     * @return array
     */
    public function toFormValues()
    {
        $return = array();
        $collection = $this->getCollection()
            ->addFieldToSelect('name')
            ->addFieldToSelect('autoresponder_id')
            ->setOrder('name', 'ASC');
        foreach ($collection as $autoresponder) {
            $return[$autoresponder->getId()] = $autoresponder->getName() . ' (ID:' . $autoresponder->getId() . ')';
        }

        return $return;
    }

    /**
     * @param $storeId
     *
     * @return mixed
     */
    protected function _getCollection($storeId)
    {

        $date = Mage::app()->getLocale()->date()->get(self::MYSQL_DATE);
        //Version Compatability
        $return = $this->getCollection()
            ->addFieldToFilter('active', 1);

        $return->getSelect()
            ->where(" FIND_IN_SET('0', store_ids) OR FIND_IN_SET(?, store_ids)", $storeId)
            ->where(" from_date <=? or from_date IS NULL ", $date)
            ->where(" to_date >=? or to_date IS NULL ", $date);


        return $return;
    }

    /**
     * @param $code
     *
     * @return mixed
     */
    public function getRedirect($code)
    {

        $events = Mage::getModel('fidelitas/events')->getCollection();

        $events->getSelect()->where('MD5(extra)=?', $code);

        return $events->getFirstItem()->getData('extra');

    }
}
