<?php


/**
 * Class Licentia_Fidelitas_Model_Egoi
 */
class Licentia_Fidelitas_Model_Egoi extends Varien_Object
{

    /**
     *
     */
    const ECOMMERCE_URL = 'https://api.egoiapp.com/';

    /**
     *
     */
    const PLUGIN_KEY = 'e419a126e087bed65ad7fe8342f2f493';
    /**
     *
     */
    const API_URL = 'http://api.e-goi.com/v2/soap.php?wsdl';


    /**
     * @var
     */
    protected $_client;
    /**
     * @var
     */
    protected $rpc;


    /**
     * @throws Zend_Soap_Client_Exception
     */
    public function _construct()
    {
        parent::_construct();

        ini_set('default_socket_timeout', 20);

        $this->rpc = new Zend_XmlRpc_Client('http://api.e-goi.com/v2/xmlrpc.php');

        $opts = array(
            'http' => array(
                'user_agent' => 'PHPSoapClient',
            ),
        );

        $context = stream_context_create($opts);

        $this->_client = new Zend_Soap_Client(self::API_URL, array("stream_context" => $context));
    }


    /**
     * @param      $number
     * @param      $message
     * @param null $storeId
     *
     * @return bool|string
     * @throws Mage_Core_Exception
     * @throws Zend_Json_Exception
     * @throws Zend_XmlRpc_Client_FaultException
     */
    public function send($number, $message, $storeId = null)
    {

        if (!$number) {
            return false;
        }

        $method = Mage::getStoreConfig('fidelitas/config/method', $storeId);

        if ($method == 'campaign') {

            $this->setData('subject', 'Sent from Magento');
            $this->setData('fromID', Mage::getStoreConfig('fidelitas/config/sender', $storeId));
            $this->setData('listID', Mage::getModel('fidelitas/lists')->getList()->getData('listnum'));
            $this->setData('message', $message);
            $this->setData('cellphone', $number);

            $client = new Zend_XmlRpc_Client('http://api.e-goi.com/v2/xmlrpc.php');
            $result = $client->call('sendSMS', array($this->getDataKey()));

            #$this->processServiceResult($this->_client->sendSMS($this->getDataKey()));
            $this->processServiceResult($result);

            Mage::log($this->getData(), 2, 'egoi-sms-data.log', true);

            if ($this->getData('id')) {
                return true;
            } else {
                return false;
            }

        } else {

            $url = 'https://www51.e-goi.com/api/public/sms/send';

            $data = array(
                "apikey"     => Mage::getStoreConfig('fidelitas/config/api_key', $storeId),
                "mobile"     => $number,
                "senderHash" => Mage::getStoreConfig('fidelitas/config/sender', $storeId),
                "message"    => $message,
            );

            $data = Zend_Json::encode($data);

            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
            curl_setopt($ch, CURLOPT_POSTFIELDS, "$data");
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            $output = curl_exec($ch);
            $http_status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);

            $result = Zend_Json::decode($output);
            if (is_array($result) && is_array($result['errors'])) {
                $result = implode(' ', $result['errors']);
            }

            return ($http_status == 200) ? true : '. Server Response: ' . $result;
        }
    }

    /**
     * @param $document
     *
     * @return bool
     * @throws Zend_Json_Exception
     */
    public function getPhone($document)
    {


        if (!is_object($document) && stripos($document, '-') === false) {
            Mage::log('Ignoring: Invalid Docuemnt', null, 'egoi.sms.txt', true);
            return false;
        }

        if (!is_object($document) && stripos($document, '-')) {
            return $this->validateNumber($document);
        }


        if ($document instanceof Mage_Sales_Model_Order) {
            $billing = $document->getBillingAddress();
        } else if ($document instanceof Mage_Customer_Model_Customer) {
            $billing = $document->getPrimaryBillingAddress();
        } elseif (is_object($document->getOrder())) {
            $billing = $document->getOrder()->getBillingAddress();
        }

        $prefix = Mage::getModel('fidelitas/subscribers')->getPrefixForCountry($billing->getCountryId());

        $cellphoneField = Mage::getStoreConfig('fidelitas/config/cellphone');
        $number = preg_replace('/\D/', '', $billing->getData($cellphoneField));
        $number = ltrim($number, $prefix);
        $number = ltrim($number, 0);

        $return = $this->validateNumber($prefix . '-' . $number);

        if (!$return) {
            Mage::log('Invalid Phone Number ' . $number . ' | ' . $document->getId(), null, 'egoi.sms.txt', true);
        }

        return $return;

    }

    /**
     * @param $number
     *
     * @return bool
     * @throws Zend_Json_Exception
     */
    public function validateNumber($number)
    {

        $url = 'https://www51.e-goi.com/api/public/sms/validatePhone/' . $number;

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($ch);
        curl_close($ch);

        $final = Zend_Json::decode($result);

        if (array_key_exists('errorCode', $final)) {
            return false;
        }

        return $number;
    }

    /**
     * @return bool
     */
    public function validateEgoiEnvironment()
    {
        $auth = Mage::getSingleton('admin/session')->getUser()->getData('fidelitasAuth');

        if ($auth === true) {
            return true;
        }

        $info = $this->getUserData()->getData();

        if (!isset($info[0]) || !isset($info[0]['user_id']) || (int)$info[0]['user_id'] == 0) {
            return false;
        }

        $account = Mage::getModel('fidelitas/account')->getAccount();

        if ((int)$account->getData('cliente_id') == 0) {

            $n = Mage::getModel('fidelitas/egoi')->getAccountDetails()->getData();
            $account->addData($n[0])->save();

            $account = Mage::getModel('fidelitas/account')->getAccount();

            if ((int)$account->getData('cliente_id') == 0) {
                return false;
            }
        }

        Mage::getSingleton('admin/session')->getUser()->setData('fidelitasAuth', true);

        return true;
    }

    /**
     * @param $data
     *
     * @return $this|void
     */
    public function formatFields($data)
    {

        if (!is_array($data)) {
            $data = array('RESULT' => $data);
        }

        if (count($data) == 1 && isset($data['ERROR'])) {
            Mage::log(serialize($data), 2, 'egoi-egoi.log');
            $data = array(0 => $data);
            $this->setData($data);

            return;
        }

        if (!array_key_exists(0, $data)) {
            $data = array(0 => $data);
        }

        foreach ($data as $key => $value) {
            $data[$key] = array_change_key_case($value, CASE_LOWER);
        }

        $this->setData($data);

        return $this;
    }


    /**
     * @return bool
     */
    public function exportBulk()
    {
        return $this->addSubscriberBulk(true);
    }

    /**
     * @param bool $generate
     * @param bool $processNumberForced
     *
     * @return bool
     */
    public function addSubscriberBulk($generate = false, $processNumberForced = false)
    {

        $file = Mage::getBaseDir('tmp') . '/egoi.txt';
        if (!is_file($file)) {
            file_put_contents($file, '0');
        }
        $lastSync = (int)file_get_contents($file);

        $meta = Mage::getModel('newsletter/subscriber')->getCollection()
            ->addFieldToFilter('subscriber_status', 1)
            ->addFieldToFilter('subscriber_id', array('gt' => $lastSync));

        $yesterday = date('Y-m-d 00:00:00', strtotime('yesterday'));

        if (!$generate) {
            $meta->addFieldToFilter('fidelitas_updated_at', array('gteq' => $yesterday));
        }

        if ($meta->getSize() == 0) {
            unlink($file);
            return true;
        }

        $list = Mage::getModel('fidelitas/lists')->getList(true);
        $extra = Mage::getModel('fidelitas/extra')->getExtra();

        $processNumber = (int)Mage::getStoreConfig('fidelitas/config/sync_number');
        if ($processNumber == 0) {
            $processNumber = 10000;
        }

        if ($processNumberForced == -1) {
            $processNumber = 100000000;
        }

        $i = 0;

        try {
            while ($i * $processNumber <= $meta->getSize()) {

                $i++;
                $core = Mage::getModel('newsletter/subscriber')
                    ->getCollection()
                    ->setPageSize($processNumber)
                    ->addFieldToFilter('subscriber_status', 1)
                    ->addFieldToFilter('subscriber_id', array('gt' => $lastSync));

                if (!$generate) {
                    $meta->addFieldToFilter('fidelitas_updated_at', array('gteq' => $yesterday));
                }

                $subscribers = array();
                $indexArray = array();
                $subI = 0;

                /** @var Mage_Newsletter_Model_Subscriber $subscriber */
                foreach ($core as $subscriber) {
                    $subI++;

                    try {
                        $data = array();

                        $data['email'] = $subscriber->getEmail();
                        $indexArray[] = 'email';

                        if (!filter_var($data['email'], FILTER_VALIDATE_EMAIL)) {
                            $subscriber->delete();
                            continue;
                        }

                        /** @var Licentia_Fidelitas_Model_Subscribers $fidelitas */
                        $fidelitas = Mage::getModel('fidelitas/subscribers')->load($subscriber->getEmail(), 'email');
                        $customer = $fidelitas->findCustomer($subscriber->getEmail(), 'email');

                        $data['birth_date'] = '';
                        $indexArray[] = 'birth_date';
                        $data['first_name'] = '';
                        $indexArray[] = 'first_name';
                        $data['last_name'] = '';
                        $indexArray[] = 'last_name';
                        $data['cellphone'] = '';
                        $indexArray[] = 'cellphone';

                        if ($customer) {
                            $data['birth_date'] = substr($customer->getData('dob'), 0, 10);
                            $data['first_name'] = $customer->getData('firstname');
                            $data['last_name'] = $customer->getData('lastname');

                            if ($customer->getData('cellphone')) {
                                $data['cellphone'] = $customer->getData('cellphone');
                            }

                            foreach ($extra as $element) {

                                if ($customer->getData($element->getData('attribute_code'))) {
                                    $data[$element->getData('extra_code')] = $customer->getData($element->getData('attribute_code'));

                                    $indexArray[] = $element->getData('extra_code');
                                    continue;
                                }

                                $billing = false;
                                if (stripos($element->getData('attribute_code'), 'addr_') !== false) {
                                    $attributeCode = substr($element->getData('attribute_code'), 5);
                                    $billing = true;
                                } else {
                                    $attributeCode = $element->getData('attribute_code');
                                }

                                if (!$customer->getData($attributeCode)) {
                                    if ($billing) {
                                        $customer = $fidelitas->findCustomer($customer->getId(), 'entity_id', $attributeCode);
                                    } else {
                                        $customer = Mage::getModel('customer/customer')->load($customer->getId());
                                    }
                                }
                                if ($customer->getData($attributeCode)) {
                                    $data[$element->getData('extra_code')] = $customer->getData($attributeCode);
                                    $indexArray[] = $element->getData('extra_code');
                                } else {
                                    $data[$element->getData('extra_code')] = '';
                                }
                            }
                        } else {
                            foreach ($extra as $element) {
                                $data[$element->getData('extra_code')] = '';
                            }
                        }

                        $extraStore = Mage::getModel('fidelitas/extra')
                            ->getExtra()
                            ->addFieldToFilter('attribute_code', 'store_id');

                        if ($extraStore->getSize() == 1) {
                            $extraStoreView = $extraStore->getFirstItem();

                            if (!isset($data[$extraStoreView->getData('extra_code')]) ||
                                $data[$extraStoreView->getData('extra_code')] == ''
                            ) {
                                if ($subscriber->getStoreId()) {
                                    $data[$extraStoreView->getData('extra_code')] = $subscriber->getStoreId();
                                } else {
                                    /** @var Mage_Sales_Model_Order $order */
                                    $order = Mage::getModel('sales/order')->getCollection()
                                        ->addFieldToFilter('customer_email', $subscriber->getEmail())
                                        ->setPageSize(1)
                                        ->getFirstItem();

                                    if ($order->getId()) {
                                        $data[$extraStoreView->getData('extra_code')] = $order->getStoreId();
                                    }
                                }
                            }
                        }

                        $data['status'] = 1;
                        $indexArray[] = 'status';
                        if ($subI == 1 && $lastSync == 0) {
                            $subscribers[] = $indexArray;
                        }

                        $lastSync = $subscriber->getId();

                        $subscribers[] = $data;
                    } catch (Exception $e) {
                    }
                }


                if ($generate === true && $processNumberForced === -1) {

                    if ($lastSync > 0) {
                        unset($subscribers[0]);
                    }

                    $fileExport = Mage::getBaseDir('tmp') . '/egoi_download.csv';
                    if (is_file($fileExport)) {
                        unlink($fileExport);
                    }
                    $fp = fopen($fileExport, 'a');

                    $i = 0;
                    foreach ($subscribers as $fields) {

                        if ($i == 0) {
                            fputcsv($fp, array_keys($fields), ';');
                        }

                        fputcsv($fp, $fields, ';');

                        $i++;
                    }

                    fclose($fp);

                    return $fileExport;

                } elseif ($generate === true) {

                    if ($lastSync > 0) {
                        unset($subscribers[0]);
                    }

                    $fileExport = Mage::getBaseDir('tmp') . '/egoi_export.csv';
                    $fp = fopen($fileExport, 'a');

                    $i = 0;
                    foreach ($subscribers as $fields) {

                        if ($i == 0) {
                            fputcsv($fp, array_keys($fields), ';');
                        }

                        fputcsv($fp, $fields, ';');

                        $i++;
                    }

                    fclose($fp);

                    if (count($subscribers) == $processNumber) {

                        $cron = Mage::getModel('cron/schedule');
                        $data['status'] = 'pending';
                        $data['job_code'] = 'fidelitas_export_bulk';
                        $data['scheduled_at'] = Mage::getSingleton('core/date')->gmtDate();
                        $data['created_at'] = Mage::getSingleton('core/date')->gmtDate();
                        $cron->setData($data)->save();

                    } else {

                        $token = md5(time());

                        $msg = 'Hi ' . Mage::getStoreConfig('trans_email/ident_sales/name') . ', ';

                        $msg .= "The exported Newsletter subscribers file is ready. 
                        <br><br>You can download it form the following link<br><br>";

                        $msg .= Mage::getUrl('egoi/download/index', array('token' => $token));

                        $msg .= "<br><br>Regards<br><br>";

                        $mail = new Zend_Mail('UTF-8');
                        $mail->setBodyHtml($msg);
                        $mail->setFrom(Mage::getStoreConfig('trans_email/ident_sales/email'),
                            Mage::getStoreConfig('trans_email/ident_sales/name'))
                            ->addTo(Mage::getStoreConfig('trans_email/ident_sales/email'),
                                Mage::getStoreConfig('trans_email/ident_sales/name'))
                            ->setSubject('E-Goi - Export Finished');
                        $mail->send();

                        file_put_contents(Mage::getBaseDir('tmp') . '/egoi_token.txt', $token);

                    }

                } else {

                    unset($subscribers[0]);

                    $params = array(
                        'apikey'        => Mage::getStoreConfig('fidelitas/config/api_key'),
                        'plugin_key'    => self::PLUGIN_KEY,
                        'listID'        => $list->getListnum(),
                        'compareField'  => 'email',
                        'operation'     => 2,
                        'autoresponder' => 0,
                        'notification'  => 0,
                        'subscribers'   => $subscribers,
                    );

                    $client = new Zend_XmlRpc_Client('http://api.e-goi.com/v2/xmlrpc.php');
                    $result = $client->call('addSubscriberBulk', array($params));

                    if (count($subscribers) == $processNumber) {
                        $cron = Mage::getModel('cron/schedule');
                        $data['status'] = 'pending';
                        $data['job_code'] = 'fidelitas_sync_bulk';
                        $data['scheduled_at'] = Mage::getSingleton('core/date')->gmtDate();
                        $data['created_at'] = Mage::getSingleton('core/date')->gmtDate();
                        $cron->setData($data)->save();
                    }
                }

                file_put_contents($file, $lastSync);
            }

        } catch (Exception $e) {
            return false;
        }

        unlink($file);
        return true;
    }


    /**
     * @return mixed
     */
    public function getDataKey()
    {

        $data = $this->getData();
        $data['apikey'] = Mage::getStoreConfig('fidelitas/config/api_key');
        $data['plugin_key'] = self::PLUGIN_KEY;

        return $data;
    }


    /**
     * @param      $result
     * @param null $index
     *
     * @return $this
     * @throws Mage_Core_Exception
     */
    public function processServiceResult($result, $index = null)
    {

        if (!is_array($result)) {
            $result = array('result' => $result);
        }

        $result = array_change_key_case($result, CASE_LOWER);

        if ($index && isset($result[$index])) {
            $result = $result[$index];
            $result = array_change_key_case($result, CASE_LOWER);
        }

        $this->setData($result);

        $additionalData = serialize(array('request' => $this->_client->getLastRequest(), 'response' => $this->_client->getLastResponse()));

        if (isset($result['error'])) {
            Mage::log($additionalData, 2, 'egoi-egoi.log', true);
            throw new Mage_Core_Exception(Mage::helper('fidelitas')->__($result['error']));
        }

        return $this;
    }

    /**
     * @return Licentia_Fidelitas_Model_Egoi
     * @throws Mage_Core_Exception
     */
    public function addExtraField()
    {
        $this->setData('type', 'texto');
        return $this->processServiceResult($this->_client->addExtraField($this->getDataKey()));
    }

    /**
     * @return $this
     */
    public function getAccountDetails()
    {
        $this->formatFields($this->_client->getClientData($this->getDataKey()));

        return $this;
    }


    /**
     * @return $this
     */
    public function getUserData()
    {
        $this->formatFields($this->_client->getUserData($this->getDataKey()));

        return $this;
    }


    /**
     * @return $this
     */
    public function getSenders()
    {
        $this->setData('channel', 'telemovel');
        $this->formatFields($this->_client->getSenders($this->getDataKey()));

        return $this;
    }


    /**
     * @param null $listnum
     *
     * @return $this
     */
    public function getLists($listnum = null)
    {

        $result = $this->_client->getLists($this->getDataKey());

        if (is_array($result)) {
            foreach ($result as $key => $value) {

                if ($listnum && $listnum != $value['listnum']) {
                    unset($result[$key]);
                }

                if (is_array($value) && (isset($value['extra_fields']) && !is_array($value['extra_fields']))) {
                    continue;
                }

                foreach ($value['extra_fields'] as $eKey => $eValue) {
                    unset($result[$key]['extra_fields'][$eKey]['listnum']);
                    unset($result[$key]['extra_fields'][$eKey]['opcoes']);
                }
            }
        }

        $this->formatFields($result);

        return $this;
    }


    /**
     * @return $this
     */
    public function getSubscriberData()
    {
        $this->formatFields($this->_client->subscriberData($this->getDataKey()));

        return $this;
    }


    /**
     * @return Licentia_Fidelitas_Model_Egoi
     * @throws Mage_Core_Exception
     */
    public function editApiCallback()
    {
        return $this->processServiceResult($this->_client->editApiCallback($this->getDataKey()));
    }


    /**
     * @return Licentia_Fidelitas_Model_Egoi
     * @throws Mage_Core_Exception
     */
    public function createList()
    {
        return $this->processServiceResult($this->_client->createList($this->getDataKey()));
    }


    /**
     * @return Licentia_Fidelitas_Model_Egoi
     * @throws Mage_Core_Exception
     */
    public function updateList()
    {
        return $this->processServiceResult($this->_client->updateList($this->getDataKey()));
    }


    /**
     * @return array|Licentia_Fidelitas_Model_Egoi
     * @throws Mage_Core_Exception
     */
    public function addSubscriber()
    {
        $this->setData('status', 1);

        if (!$this->getData('email')) {
            return array('error' => 'EMPTY_EMAIL');
        }

        return $this->processServiceResult($this->_client->addSubscriber($this->getDataKey()));
    }


    /**
     * @return Licentia_Fidelitas_Model_Egoi
     * @throws Mage_Core_Exception
     */
    public function editSubscriber()
    {

        if (!$this->getData('uid')) {
            return array('error' => 'EMPTY_UID');
        }

        $data = $this->getDataKey();
        $exist = $this->getSubscriberData($this->getDataKey())->getData();
        $this->setData($data);

        if (is_array($exist) && isset($exist['0']['ERROR'])) {
            return $this->addSubscriber();
        }

        return $this->processServiceResult($this->_client->editSubscriber($data));
    }


    /**
     * @return array|Licentia_Fidelitas_Model_Egoi|void
     * @throws Mage_Core_Exception
     */
    public function removeSubscriber()
    {
        $result = Mage::getModel('fidelitas/egoi')
            ->setData('listID', $this->getData('listID'))
            ->setData('subscriber', $this->getSubscriber())
            ->getSubscriberData()
            ->getData();

        if (is_array($result) && $result[0]['subscriber']['STATUS'] != 2) {
            return $this->processServiceResult($this->_client->removeSubscriber($this->getDataKey()));
        }

        if ($this->getData('inCron')) {
            return;
        }

        return array('error' => Mage::helper('fidelitas')->__('Subscriber not found or action not allowed'));
    }


    /**
     * @param null $apiKey
     *
     * @return $this
     * @throws Mage_Core_Exception
     */
    public function checkLogin($apiKey = null)
    {

        $data = $this->getDataKey();
        if ($apiKey) {
            $data['apikey'] = $apiKey;
        }
        $this->processServiceResult($this->_client->checklogin($data));

        return $this;
    }


    /**
     */
    public function sync()
    {
        $account = Mage::getModel('fidelitas/account')->getAccount();
        $key = Mage::getStoreConfig('fidelitas/config/api_key');

        if (!$key) {
            return;
        }

        $models = array('subscribers', 'account');
        foreach ($models as $sync) {
            Mage::getModel('fidelitas/' . $sync)->cron();
        }

        $account->setData('cron', 0)->save();
    }


    /**
     */
    public function syncm()
    {
        $account = Mage::getModel('fidelitas/account')->getAccount();

        if ($account->getCron() == 3) {
            Mage::registry('fidelitas_first_run', true);
        }

        Mage::getModel('fidelitas/egoi')->sync();

    }

    /**
     * @param $method
     * @param $arguments
     *
     * @return array
     */
    function buildParams($method, $arguments)
    {

        return array(
            "method"          => $method,
            "functionOptions" => $arguments,
            "type"            => "json",
        );
    }


    /**
     * @param  $storeId
     *
     * @return bool|string
     */
    public function createCatalog($storeId)
    {

        /** @var Mage_Core_Model_Store $store */
        $store = Mage::getModel('core/store');
        $store->load($storeId);

        $data = [];
        $data['title'] = 'Magento - ' . $store->getName() . ' [' . $store->getId() . ']';
        $data['currency'] = $store->getBaseCurrencyCode();
        $data['language'] = substr(
            Mage::getStoreConfig('general/locale/code', $store->getId()),
            0,
            2
        );

        $url = self::ECOMMERCE_URL . 'catalogs';

        $ch = curl_init($url);
        curl_setopt(
            $ch,
            CURLOPT_HTTPHEADER,
            [
                'Content-Type: application/json',
                'Apikey: ' . Mage::getStoreConfig('fidelitas/config/api_key'),
            ]
        );
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        $result = curl_exec($ch);
        curl_close($ch);

        return json_decode($result, true);

    }

    public function checkEcommerceCatalogs()
    {

        $stores = $this->getEcommerceCatalogs();
        $allStores = array_keys(Mage::app()->getStores());

        $core = Mage::getModel('core/config');
        foreach ($allStores as $storeId) {

            $catalogIdForStore = Mage::getStoreConfig('fidelitas/products/catalog_id', $storeId);
            if (!$catalogIdForStore || !array_key_exists($catalogIdForStore, $stores)) {

                $result = $this->createCatalog($storeId);

                $core->saveConfig('fidelitas/products/catalog_id', $result['catalog_id'], 'stores', $storeId);
                $core->saveConfig('fidelitas/products/enable', 1, 'stores', $storeId);

                Mage::getConfig()->reinit();
                Mage::app()->reinitStores();

            }
        }
    }

    /**
     *
     */
    public function getEcommerceCatalogs()
    {

        $url = self::ECOMMERCE_URL . 'catalogs';

        $ch = curl_init($url);
        curl_setopt(
            $ch,
            CURLOPT_HTTPHEADER,
            [
                'Content-Type: application/json',
                'Apikey: ' . Mage::getStoreConfig('fidelitas/config/api_key'),
            ]
        );
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($ch);
        curl_close($ch);

        $final = json_decode($result, true);


        $catalogs = [];

        if ($final['total_items'] > 0) {
            foreach ($final['items'] as $info) {
                $catalogs[$info['catalog_id']] = $info['title'] . ' - ' . $info['language'] . ' - ' . $info['currency'];
            }
        }

        return $catalogs;

    }

    /**
     *
     */
    public function getActiveEcommerceCatalogs()
    {

        $stores = $this->getEcommerceCatalogs();
        $return = [];
        $allStores = Mage::app()->getStores();

        foreach ($allStores as $store) {

            $catalogIdForStore = Mage::getStoreConfig('fidelitas/products/catalog_id', $store->getId());
            if ($catalogIdForStore && array_key_exists($catalogIdForStore, $stores)) {
                $return[$store->getCode()] = $stores[$catalogIdForStore];

            }
        }

        return $return;

    }

    /**
     * @param int $storeId
     *
     * @return bool
     */
    public function buildProductsFeed($storeId = 1)
    {
        Mage::app()->setCurrentStore($storeId);

        $attributesInfo = Mage::getStoreConfig('fidelitas/products', $storeId);
        $catalogId = $attributesInfo['catalog_id'];

        if (!isset($attributesInfo['enable']) || !$attributesInfo['enable']) {
            return false;
        }

        /** @var Mage_Catalog_Model_Category $category */
        $category = Mage::getModel('catalog/category');
        $tree = $category->getTreeModel();
        $tree->load();

        $ids = $tree->getCollection()->addIsActiveFilter()->getAllIds();
        $cnt = 0;
        $categories = [];
        $catInfo = [];
        if ($ids) {

            foreach ($ids as $k => $id) {
                $category->load($id);
                $categories[$id]['name'] = $category->getName();
                $categories[$id]['path'] = $category->getPath();
            }

            foreach ($ids as $id) {

                $path = explode('/', $categories[$id]['path']);
                $string = '';
                foreach ($path as $pathId) {
                    if (isset($categories[$pathId])) {
                        $string .= $categories[$pathId]['name'] . ' / ';
                        $cnt++;
                    }
                }

                $string = str_replace('Root Catalog / Default Category / ', '', $string);
                $string = str_replace('Default Category / ', '', $string);
                $catInfo[$id] = rtrim($string, ' / ');
            }
        }

        $collection = Mage::getModel('catalog/product')
            ->getCollection()
            ->addAttributeToSelect('*')
            ->joinField(
                'qty',
                'cataloginventory_stock_item',
                'qty',
                'product_id=entity_id',
                '{{table}}.stock_id=1',
                'left'
            );

        $collection->addAttributeToSelect('price')
            ->addAttributeToFilter('status', 1)
            ->addAttributeToFilter('visibility', 4)
            ->addAttributeToFilter('type_id', 'simple')
            ->addAttributeToFilter('qty', ['gt' => 0]);

        $return = [];

        /** @var Mage_Catalog_Model_Product $product */
        foreach ($collection as $product) {


            $imgUrl = '';

            try {

                $parents = Mage::getModel('catalog/product_type_configurable')->getParentIdsByChild($product->getId());
                if ($parents) {
                    $imgUrl = Mage::getModel('catalog/product')->load($parents[0])->getImageUrl();
                } else {
                    $imgUrl = $product->getImageUrl();
                }

            } catch (Exception $e) {

            }

            $return[$product->getId()]['product_identifier'] = $product->getId();
            $return[$product->getId()]['sku'] = $product->getSku();
            $return[$product->getId()]['name'] = $product->getName();
            $return[$product->getId()]['description'] = $product->getShortDescription();
            $return[$product->getId()]['price'] = number_format($product->getPrice(), 2);
            $return[$product->getId()]['image_link'] = $imgUrl;
            $return[$product->getId()]['link'] = $product->getProductUrl();

            if ($product->getFinalPrice() < $product->getPrice()) {
                $return[$product->getId()]['sale_price'] = number_format($product->getFinalPrice(), 2);
            }

            $cats = $product->getCategoryIds();

            foreach ($cats as $cat) {
                if (isset($catInfo[$cat])) {
                    $return[$product->getId()]['categories'][] = $catInfo[$cat];
                }
            }

            $related = $product->getRelatedProductCollection();

            foreach ($related as $info) {

                $return[$product->getId()]['related_product'][] = $info->getSku();

            }

            $attributes = $product->getAttributes();

            foreach ($attributes as $attribute) {

                if (!in_array($attribute->getAttributeCode(), $attributesInfo)) {
                    continue;
                }

                if ($attribute->getSource()) {

                    $value = $product->getResource()
                        ->getAttribute($attribute->getAttributeCode())
                        ->getFrontend()
                        ->getValue($product);

                    $return[$product->getId()][array_search(
                        $attribute->getAttributeCode(),
                        $attributesInfo
                    )] = (string)$value;

                }

            }

        }

        $return = ['products' => $return];

        $url = self::ECOMMERCE_URL . 'catalogs/' . $catalogId . '/products/actions/import';

        $ch = curl_init($url);
        curl_setopt(
            $ch,
            CURLOPT_HTTPHEADER,
            [
                'Content-Type: application/json',
                'Apikey: ' . Mage::getStoreConfig('fidelitas/config/api_key'),
            ]
        );
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($return));
        $result = curl_exec($ch);
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        return ($httpcode < 300);

    }

    function syncCatalogs()
    {

        $allStores = array_keys(Mage::app()->getStores());

        foreach ($allStores as $storeId) {
            $this->buildProductsFeed($storeId);

        }
    }
}
