<?php

/**
 * Class Licentia_Fidelitas_Model_Observer
 */
class Licentia_Fidelitas_Model_Observer
{

    /**
     * Add order information into Egoimmerce block to render on checkout success pages
     *
     * @param Varien_Event_Observer $observer
     */
    public function setEgoimmerceOnOrderSuccessPageView(Varien_Event_Observer $observer)
    {
        $orderIds = $observer->getEvent()->getOrderIds();
        if (empty($orderIds) || !is_array($orderIds)) {
            return;
        }
        $block = Mage::app()->getFrontController()->getAction()->getLayout()->getBlock('egoimmerce');
        if ($block) {
            $block->setOrderIds($orderIds);
        }
    }


    /**
     * @param $event
     *
     * @return void
     */
    public function addToAutoList($event)
    {
        /** @var Mage_Sales_Model_Order $order */
        $order = $event->getEvent()->getOrder();

        try {
            if (Mage::getStoreConfig('fidelitas/config/auto')) {
                Mage::getModel('newsletter/subscriber')->subscribe($order->getCustomerEmail());
            }
        } catch (Exception $e) {
            Mage::logException($e);
        }
    }

    /**
     * @param $event
     *
     * @return void
     */
    public function finalizeConversion($event)
    {
        /** @var Mage_Sales_Model_Order $order */
        $order = $event->getEvent()->getOrder();

        try {

            if ($cookie = Mage::getSingleton('core/cookie')->get('egoi_conv')) {

                $cookie = json_decode($cookie, true);

                Mage::getModel('fidelitas/conversions')
                    ->setData(array(
                        'created_at'   => $order->getCreatedAt(),
                        'email'        => $order->getCustomerEmail(),
                        'order_id'     => $order->getIncrementId(),
                        'order_amount' => $order->getBaseGrandTotal(),
                    ))
                    ->addData($cookie)
                    ->save();

            }


        } catch (Exception $e) {
            Mage::logException($e);
        }
    }

    /**
     * @param $event
     */
    public function pageseguroOrder($event)
    {
        /** @var Mage_Core_Controller_Varien_Action $action */
        $action = $event->getControllerAction();
        $redirect = $action->getRequest()->getParam('redirect');

        $orderId = Mage::getModel('core/session')->getData('egoi_pagseguro');
        $order = Mage::getModel('sales/order')->load($orderId);

        if ($order->getId()) {
            Mage::getModel('fidelitas/autoresponders')->newOrder($order, $redirect);
            Mage::getModel('core/session')->setData('egoi_pagseguro', null);
        }

    }


    /**
     * @param $event
     */
    public function conversionsStart($event)
    {
        /** @var Mage_Core_Controller_Varien_Action $action */
        $action = $event->getControllerAction();
        $source = $action->getRequest()->getParam('utm_source');

        if ($source == 'e-goi') {
            Mage::getSingleton('core/cookie')->set('egoi_conv', json_encode($action->getRequest()->getParams()), 3600 * 24 * 3);
        }

    }


}
