<?php

/**
 * Licentia Fidelitas - Advanced Email and SMS Marketing Automation for E-Goi
 *
 * NOTICE OF LICENSE
 * This source file is subject to the Creative Commons Attribution-NonCommercial 4.0 International
 * It is available through the world-wide-web at this URL:
 * http://creativecommons.org/licenses/by-nc/4.0/
 *
 * @title      Advanced Email and SMS Marketing Automation
 * @category   Marketing
 * @package    Licentia
 * @author     Bento Vilas Boas <bento@licentia.pt>
 * @copyright  Copyright (c) 2012 Licentia - http://licentia.pt
 * @license    Creative Commons Attribution-NonCommercial 4.0 International
 */
class Licentia_Fidelitas_JsonController extends Mage_Core_Controller_Front_Action
{

    public function indexAction()
    {

        $categories = $this->getRequest()->getParam('categories');
        $categories = explode(',', $categories);
        $categories = array_map('trim', $categories);
        $categories = array_filter($categories);

        $storeId = $this->getRequest()->getParam('store_id');

        $ids = $this->getRequest()->getParam('ids');
        $ids = explode(',', $ids);
        $ids = array_map('trim', $ids);
        $ids = array_filter($ids);

        $skus = $this->getRequest()->getParam('skus');
        $skus = explode(',', $skus);
        $skus = array_map('trim', $skus);
        $skus = array_filter($skus);

        /** @var Mage_Catalog_Model_Resource_Product_Collection $collection */
        $collection = Mage::getModel('catalog/product')->getCollection()
            ->addAttributeToSelect('*')
            ->joinField(
                'qty',
                'cataloginventory_stock_item',
                'qty',
                'product_id=entity_id',
                '{{table}}.stock_id=1',
                'left'
            );

        foreach ($categories as $category) {
            $collection->addCategoryFilter(Mage::getModel('catalog/category')->load($category));
        }

        $collection->addAttributeToSelect('price');
        $collection->joinAttribute('status', 'catalog_product/status', 'entity_id', null, 'inner')
            ->addAttributeToFilter('status', 1)
            ->addAttributeToFilter('visibility', 4)
            ->setCurPage($this->getRequest()->getParam('start', 0))
            ->setPageSize($this->getRequest()->getParam('limit', 20));

        if (count($ids) > 0) {
            $collection->addAttributeToFilter('entity_id', ['in' => $ids]);
        }

        if (count($skus) > 0) {
            $collection->addAttributeToFilter('sku', ['in' => $skus]);
        }

        if ($storeId) {
            $collection->setStoreId($storeId);
        }

        $return = [];

        foreach ($collection as $product) {

            if (!$product->isSalable()) {
                continue;
            }

            $cats = $product->getCategoryCollection()->addAttributeToSelect('name');
            $rels = $product->getRelatedProductCollection();
            $ups = $product->getUpSellProductCollection();

            $return = [
                'id'                => $product->getId(),
                'name'              => $product->getName(),
                'sku'               => $product->getSku(),
                'short_description' => $product->getShortDescription(),
                'price'             => Mage::helper('core')->currency($product->getPrice(), true, false),
                'url'               => $product->getProductUrl(),
                'thumbnail'         => (string)Mage::helper('catalog/image')->init($product, 'thumbnail')
                    ->constrainOnly(false)
                    ->keepAspectRatio(true)
                    ->keepFrame(true)
                    ->resize(150, 150),
                'small_image'       => (string)Mage::helper('catalog/image')->init($product, 'small_image')
                    ->constrainOnly(false)
                    ->keepAspectRatio(true)
                    ->keepFrame(true)
                    ->resize(300, 300),
                'image'             => (string)Mage::helper('catalog/image')->init($product, 'image')
                    ->constrainOnly(false)
                    ->keepAspectRatio(true)
                    ->keepFrame(true)
                    ->resize(600, 600),
                'large_image'       => (string)Mage::helper('catalog/image')->init($product, 'image'),
            ];

            $specialPrice = $product->getSpecialPrice();
            if ($product->getPrice() > $product->getFinalPrice()) {
                $specialPrice = $product->getFinalPrice();
            }

            $return['special_price'] = $specialPrice > 0 ? Mage::helper('core')->currency($specialPrice, true, false) : '';
            #$return['special_to_date'] = $product->getData('special_to_date');


            foreach ($cats as $cat) {
                $return['categories'][$cat->getId()] = $cat->getName();
            }

            foreach ($rels as $rel) {
                $return['related'][$rel->getId()] = $rel->getId();
            }

            foreach ($ups as $up) {
                $return['upsell'][$up->getId()] = $up->getId();
            }

            $return1['item'][] = $return;
            unset($return[$product->getId()]);

        }

        echo json_encode(['items' => $return1]);
        die();

    }

}
